/*
  Copyright (C) 1999 Rainer Maximini

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package
*/

#include <stdio.h>
#include <stdlib.h>
#include <qstring.h>
#include <unistd.h>
#include "mp3.h"

int genre_largest=80;

char *genres[] = {    
  "Blues",    "Classic Rock",    "Country",    "Dance",
  "Disco",    "Funk",    "Grunge",    "Hip-Hop",
  "Jazz",    "Metal",    "New Age",    "Oldies",
  "Other",    "Pop",    "R&B",    "Rap",
  "Reggae",    "Rock",    "Techno",    "Industrial",
  "Alternative",    "Ska",    "Death Metal",    "Pranks",
  "Soundtrack",    "Euro-Techno",    "Ambient",    "Trip-Hop",
  "Vocal",    "Jazz+Funk",    "Fusion",    "Trance",
  "Classical",    "Instrumental",    "Acid",    "House",
  "Game",    "Sound Clip",    "Gospel",    "Noise",
  "AlternRock",    "Bass",    "Soul",    "Punk",
  "Space",    "Meditative",    "Instrumental Pop",    "Instrumental Rock",
  "Ethnic",    "Gothic",    "Darkwave",    "Techno-Industrial",
  "Electronic",    "Pop-Folk",    "Eurodance",    "Dream",
  "Southern Rock",    "Comedy",    "Cult",    "Gangsta",
  "Top 40",    "Christian Rap",    "Pop/Funk",    "Jungle",
  "Native American",    "Cabaret",    "New Wave",    "Psychadelic",
  "Rave",    "Showtunes",    "Trailer",    "Lo-Fi",
  "Tribal",    "Acid Punk",    "Acid Jazz",    "Polka",
  "Retro",    "Musical",    "Rock & Roll",    "Hard Rock",
  "Unknown"};


Tag::Tag() {
  title[0] = 0;
  artist[0] = 0;
  album[0] = 0;
  year[0] = 0;
  comment[0] = 0;
  genre[0] = 0;
  gennum = -1;
}

void Tag::clear(){
  *title   = '\0';
  *artist  = '\0';
  *album   = '\0';
  *year    = '\0';
  *comment = '\0';
  *genre   = '\0';
  gennum = -1;
}



void Tag::safecopy(char *to, char *from, int maxlen){
  int where;
  strncpy(to, from, maxlen);
  to[maxlen] = 0;
  for (where = maxlen - 1; ((where >= 0) && (to[where] == ' ')); where--) {
    to[where] = 0;
  }
}

void Tag::spacecopy(char *to, char *from, int maxlen){
    int where;
    strncpy(to, from, maxlen);
    for (where = maxlen - 1; ((where >= 0) && (to[where] == 0)); where--) {
        to[where] = ' ';
    }
}    

bool Tag::scan(FILE * file){
  int temp;
  tag song;
  fseek(file, -128, SEEK_END);
  temp = fread(&song, 128, 1, file);
  
  if (!strncmp(song.tag, "TAG", 3)) {
    safecopy(title, song.title, 30);
    safecopy(artist, song.artist, 30);
    safecopy(album, song.album, 30);
    safecopy(year, song.year, 4);
    safecopy(comment, song.comment, 30);
    gennum = song.genre & 0xFF;
    if ((gennum < 0) || (gennum >= genre_largest)) gennum = genre_largest;
    strcpy(genre, genres[gennum]);
    return (1);
  } 
  else return (0);
}


bool Tag::saveTags(FILE * file){
  size_t temp;
  tag song;
  
  spacecopy(song.title, title, 30);
  spacecopy(song.artist, artist, 30);
  spacecopy(song.album, album, 30);
  spacecopy(song.year, year, 4);
  spacecopy(song.comment, comment, 30);
  
  if (gennum >= genre_largest) song.genre = 0;
  else {
    temp = gennum;
    song.genre = (unsigned char) temp;
  }

  fseek(file, -128, SEEK_END);
  temp = fread(&song, 3, 1, file);
  if (!strncmp(song.tag, "TAG", 3)) fseek(file, -128, SEEK_END);
  else fseek(file, 0, SEEK_END);
  
  strncpy(song.tag, "TAG", 3);
  temp = fwrite(&song, 128, 1, file);
  if (temp == 1) return (true);
  perror("SAVE");
  return (false);
}
             




